var b_medians, b_values, b_chunks, b_buckets;

(function(window, document, Chartist) {
    'use strict';

    Chartist.plugins = Chartist.plugins || {};
    Chartist.plugins.cgGridOnTop = function() {
        return function cgGridOnTop(chart) {
            if(chart instanceof Chartist.Line || Chartist.Bar) {
                chart.on('created', function() {
                    var layerGrid = chart.svg._node.children[0];
                    var lastChild = chart.svg._node.children[1];

                    lastChild.parentNode.insertBefore(layerGrid, lastChild.nextSibling);
                });
            }
        };
    };
}(window, document, Chartist));

(function (Dashboard, $, Chartist, d3, moment, undefined) {
    'use strict';

    if ($.cookie('rtduration'))
        Dashboard.duration = parseInt($.cookie('rtduration'))

    if ($.cookie('rtperiod'))
        Dashboard.period = parseInt($.cookie('rtperiod'))

    Dashboard.req_url = Dashboard.req_url || 'realtime.php';
    Dashboard.req_qs = Dashboard.req_qs || 'requests';
    Dashboard.maxhist = Dashboard.maxhist || 601;
    Dashboard.divisor = parseInt(Dashboard.divisor || 3);
    Dashboard.duration = parseInt(Dashboard.duration) || 300;
    Dashboard.period = Dashboard.period || 1000;
    Dashboard.chart_options = Dashboard.chart_options || {};
    Dashboard.chart_type = Dashboard.chart_type || Chartist.Line;
    Dashboard.time_bucket_ms = Dashboard.time_bucket_ms || 1000;
    Dashboard._created = false;
    Dashboard._started = false;
    Dashboard._drawn = false;
    Dashboard.paused = $.cookie('rtpaused') == 'true'
    Dashboard.line_smooth = Dashboard.duration >= 300 ? Chartist.Interpolation.monotoneCubic() : Chartist.Interpolation.simple()

    $('button[name=rtpaused]')
        .data('paused', Dashboard.paused)
        .addClass(Dashboard.paused ? 'btn-green-outline' : 'btn-red-outline')
        .removeClass('btn-grey-outline')
        .children('i.fa')
          .removeClass('fa-pause')
          .addClass(Dashboard.paused ? 'fa-play' : 'fa-pause');

    function date_inter(value, i) {
        var step = Math.ceil(Dashboard.duration / Dashboard.divisor),
            offset = -Dashboard.duration + i * step
        if (i == Dashboard.divisor)
            return 'Now'
        else
            return '-' + moment(new Date(Date.now() + offset * 1000)).fromNow(true)
    }

    Date.prototype._toString = Date.prototype.toString;
    Date.prototype.toString = function() {
      var s = this._toString();
      return s.substr(16, 8);
    }

    var stale, wait, xhr, chart,
        last = {},
        series = {
          cached: [],
          backend: [],
          ratelimit: [],
          firewall: [],
          synthetic: []
        },
        delays = 10,
        durations = 250,
        delay = 250,
        seq_i = false,
        seq_n = false,
        seq_l = 0,
        prior,
        no_animate = false,
        legend_options = {
          clickable: false,
          legendNames: {
            cached: 'Served from Cache',
            backend: 'Served from Backend',
            ratelimit: 'Ratelimit Exceeded',
            firewall: 'Blocked by Firewall',
            synthetic: 'Response Synthesized'
          }
        },
        chart_options = {
          height: 255,
          fullWidth: true,
          showArea: true,
          showLine: false,
          showPoint: false,
          stackBars: true,
          chartPadding: {
            top: 10,
            right: 15,
            bottom: 15,
            left: 15
          },
          plugins: [
            Chartist.plugins.legend(legend_options),
            Chartist.plugins.cgGridOnTop(),
          ],
          axisX: {
            type: Chartist.FixedScaleAxis,
            divisor: Dashboard.divisor,
            labelInterpolationFnc: date_inter,
            offset: 10,
            labelOffset: {
              x: -10,
              y: 0
            },
            onlyInteger: true,
          },
          axisY: {
            type: Chartist.AutoScaleAxis,
            onlyInteger: true,
            referenceValue: 1,
            position: 'end',
            offset: 15,
            labelOffset: {
              x: 0,
              y: 4
            },
            scaleMinSpace: 40
          },
          lineSmooth: Dashboard.line_smooth
        }

    Dashboard.chart_runtime = function(d) {
        if (typeof d !== 'object')
          return;

        !d.handling || create_donut('#donut_handling', d.handling),
        !d.storage || create_donut('#donut_storage', d.storage),
        !d.bandwidth || create_donut('#donut_bandwidth', d.bandwidth)
    }

    function xdate(d) {
      return {
        x: new Date(d.x),
        y: d.y
      };
    }

    Dashboard.chart_requests = function(d) {
        if (typeof d !== 'object')
          return;

        var data = d.series || {},
            now = d.timestamp,
            points = {};

        for (var name in data) {
          for (var i in data[name]) {
            var point = data[name][i],
                last_x = last[name] || 0;

            if (points[point.x] || point.x <= last_x || point.x >= now)
              continue;

            points[point.x] = {y: 0, x: point.x};
          }
        }

        for (var name in data) {
          var buf = Object.assign({}, points);

          var last_x = last[name] || 0,
              start = 0,
              end = data[name].length - 1;

          for (var i in data[name]) {
            var point = data[name][i];
            if (buf[point.x])
              buf[point.x] = point;
          }

          buf = Object.values(buf);
          buf.sort(function (a, b) {
            return a.x - b.x;
          })

          series[name].push.apply(series[name], buf.map(xdate));

          series[name].splice(0, series[name].length - Dashboard.maxhist);

          if (buf.length)
            last[name] = buf[buf.length - 1].x;
        }

        Dashboard.redraw()
    }

    Dashboard.data = function(count) {
      var count = count || Dashboard.maxhist,
          data = {};

      Object.keys(series).forEach(function (name) {
        data[name] = series[name].slice(-count);
      });

      return data;
    }

    Dashboard.done = function(d) {
        if (typeof d !== 'object')
            return;

        if (d.requests)
            Dashboard.chart_requests(d.requests);

        if (d.runtime)
            Dashboard.chart_runtime(d.runtime);
    }

    Dashboard.fail = function(xhr, msg) {
      var traffic = $('.realtime_chart .chart'),
          donuts = $('.realtime_donuts'),
          message = traffic.children('.message');

      if (donuts.find('svg').length == 0)
        donuts.closest('section').toggle(false);

      Dashboard.stop();
      Dashboard.redraw();

      traffic.addClass('error');
      donuts.addClass('error');

      if (xhr && xhr.status == 401)
        message.text("WebHost Manager session expired, please reload.");
      else if (xhr)
        message.text("Failed reading web traffic statistics. (HTTP " + xhr.status + ")");
      else
        message.text(msg);
    }

    Dashboard.stop = function() {
      !Dashboard.poll || clearInterval(Dashboard.poll);
      Dashboard._started = false;
    }

    Dashboard.do = function(qs, done, fail) {
      if (!Dashboard.abort && (!xhr || xhr.readyState == 4)) {

        var since = Math.min.apply(Math, Object.values(last))
        if (isFinite(since))
          qs += '&since=' + since;

        xhr = $.getJSON(Dashboard.req_url, qs)
               .done(done || $.noop)
               .fail(fail || $.noop);
      }
    }

    Dashboard.start = function(period, pollqs, initqs) {
      if (Dashboard._started == true)
        return false;

      Dashboard._started = true;

      if (period)
        Dashboard.period = period;

      if (initqs)
          Dashboard.do(initqs, Dashboard.done, Dashboard.fail);

      if (Dashboard.period > 0) {
        if (Dashboard.poll)
          clearInterval(Dashboard.poll);

        Dashboard.poll = setInterval(
          function() {
            Dashboard.do(pollqs, Dashboard.done, Dashboard.fail);
          },
          Dashboard.period
        );
      }
    }

    Dashboard.redraw = function() {
      if (document.hidden)
          return;

      var data = {},
          below;


      for (var name in series) {
        data[name] = series[name].slice(-Dashboard.duration).map(
          function(point, i) {
            var y = below && data[below][i] ? data[below][i].y + point.y : point.y;
            // var add = (below && data[below] && data[below][i]) ? data[below][i].y : 0;

            return {x: point.x, y: y}
          }
        );

        below = name;
      }

      var data2 = {series: []}
      for (var name in data) {
        data2.series.push({
          data: data[name].slice(-Dashboard.duration),
          className: name
        });
      }

      data2.series.reverse();

      if (chart == undefined) {
          chart = new Dashboard.chart_type('.ct-chart', data2, chart_options);
          chart.on('draw', draw);
          chart.on('created', created);
          chart.duration = Dashboard.duration;
      } else {
          chart.update(data2, {lineSmooth: Dashboard.line_smooth}, true);
      }

      Dashboard.stale = false;
    }

    Dashboard.do(
      'fields[runtime]&fields[requests]=' + Dashboard.maxhist,
      Dashboard.done,
      Dashboard.fail
    );

    $(window).on('unload beforeunload', function() {
        Dashboard.abort = true;
        // !Dashboard.poll || clearInterval(Dashboard.poll)
        // !Dashboard.wait || Dashboard.req.abort()
    })

    $(window).on('visibilitychange', function() {
        if (!document.hidden)
            Dashboard.redraw()
    })

    $('select[name=rtduration]').on('change', function() {
      Dashboard.duration = parseInt($(this).find(':selected').val()) || 300;
      Dashboard.line_smooth = Dashboard.duration >= 300 ? Chartist.Interpolation.step() : Chartist.Interpolation.simple();

      $.cookie('rtduration', Dashboard.duration)
      Dashboard.redraw()
    })

    $('button[name=rtpaused]').on('click', function() {
      var button = $(this),
          paused = button.data('paused') != true;

      Dashboard.paused = paused;

      if (paused)
        Dashboard.stop();
      else
        Dashboard.start(Dashboard.period, "fields[requests]")

      $.cookie('rtpaused', paused)

      button
        .data('paused', paused)
        .removeClass('btn-red-outline btn-green-outline btn-grey-outline')
        .addClass(paused ? 'btn-green-outline' : 'btn-red-outline')
        .children('i.fa')
          .removeClass('fa-play fa-pause')
          .addClass(paused ? 'fa-play' : 'fa-pause')
    })

    function update_donut(path, new_data, legend) {
      path.data(pie(new_data));
      path.transition().duration(750).attrTween("d", arc_tween);
      legend.selectAll('.legend_desc').text(
        function(d) {
          for (var i = 0; i < dataset1.length; i++) {
            if (dataset1[i].label == d) {
              return dataset1[i].desc;
            }
          }
          return '';
        }
      );
    }

    function created() {
      if (Dashboard._created == true)
        return;

      Dashboard._created = true;

      if (!Dashboard.paused)
        Dashboard.start(Dashboard.period, 'fields[requests]');
    }

    function draw(d) {
      if (Dashboard._drawn == true)
        return;

      Dashboard._drawn = true;

      if (d.type === 'bar') {
        if (!seq_n || seq_i != d.seriesIndex) {
            seq_n = d.series.length
            seq_l = d.series.length
            seq_i = d.seriesIndex
        }

        seq_n--;

        d.element.animate({
          opacity: {
            begin: 0 + seq_n * 3,
            dur: 250,
            from: 0,
            to: 0.8
          }
        });
    }
    else if (d.type === 'label' && d.axis === 'x') {
        d.element.animate({
          y: {
            begin: delay,
            dur: durations,
            from: d.y + 100,
            to: d.y,
            easing: 'easeOutQuart'
          }
        });
    }
    else if (d.type === 'label' && d.axis === 'y') {
        d.element.animate({
          x: {
            begin: delay,
            dur: durations,
            from: d.x - 100,
            to: d.x,
            easing: 'easeOutQuart'
          }
        });
    }
    else if (d.type === 'point') {
        d.element.animate({
          x1: {
            begin: delay,
            dur: durations,
            from: d.x - 10,
            to: d.x,
            easing: 'easeOutQuart'
          },
          x2: {
            begin: delay,
            dur: durations,
            from: d.x - 10,
            to: d.x,
            easing: 'easeOutQuart'
          },
          opacity: {
            begin: delay,
            dur: durations,
            from: 0,
            to: 1,
            easing: 'easeOutQuart'
          }
        });
    }
    else if (d.type === 'grid') {
        var pos1Animation = {
          begin: delay,
          dur: durations,
          from: d[d.axis.units.pos + '1'] - 30,
          to: d[d.axis.units.pos + '1'],
          easing: 'easeOutQuart'
        };

        var pos2Animation = {
          begin: delay,
          dur: durations,
          from: d[d.axis.units.pos + '2'] - 100,
          to: d[d.axis.units.pos + '2'],
          easing: 'easeOutQuart'
        };

        var animations = {};
        animations[d.axis.units.pos + '1'] = pos1Animation;
        animations[d.axis.units.pos + '2'] = pos2Animation;
        animations['opacity'] = {
          begin: delay,
          dur: durations,
          from: 0,
          to: 1,
          easing: 'easeOutQuart'
        };

        d.element.animate(animations);
      }
    }
  }
)(window.Dashboard = window.Dashboard || {}, jQuery, Chartist, d3, moment);


function create_donut(container, metadata, sub_metadata) {
    if (metadata == undefined)
        return

    var height = 290, width = 195;
    var radius = 75;
    var margin = 2;
    var thickness = 25;
    var offset_inner_y = 2;
    var offset_info_x = 1;
    var inner_title_y = 12;

    var legend_rect_size = 8,
        legend_rect_left_margin = 6,
        legend_rect_right_margin = 6

    var legend_title_height = 30,
        legend_title_top_margin = 10,
        legend_title_bottom_margin = 15

    var legend_row_height = 18;
    var legend_spacing = 2;
    var legend_btm_margin = 15;
    var donuts = [];
    var svg, svg_inner, legend, x_trans, y_trans,
        svg_main, values_main, colors_main, pie_main, arc_main, path_main, pie_main,
        svg_sub, values_sub, colors_sub, pie_sub, arc_sub, path_sub

    values_main = metadata.data.values
    colors_main = d3.scale.ordinal().range(metadata.data.colors)

    width = width + 2 * margin
    x_trans = width / 2
    y_trans = radius + margin

    if (sub_metadata != undefined) {
        values_sub = sub_metadata.data.values
        colors_sub = d3.scale.ordinal().range(sub_metadata.data.colors)

        height += (values_sub.length * legend_rect_size) + (values_sub.length - 1) * legend_spacing
        thickness = thickness / values_sub.length
    }

    svg = d3.select(container).append('svg')
                            .attr('width', width)
                            .attr('height', height)
                            .append('g')

    svg_main = svg.append('g')
                  .attr('transform', translate(width / 2, radius + margin))

    arc_main = d3.svg.arc()
                     .innerRadius(radius - thickness).outerRadius(radius)

    pie_main = d3.layout.pie()
                        .value(function(d) { return d.count; })
                        .sort(null)

    var sum = 0;
    for (var i in values_main)
        sum += values_main[i].count;
    if (sum === 0)
        values_main[0].count = 1;

    path_main = svg_main.selectAll('path')
                   .data(pie_main(values_main))
                   .enter().append('path')

    path_main.attr('fill', function(d, i) { return colors_main(d.data.label); })
             .attr("d", arc_main)
             .each(function(d) { this._arc = arc_main; this._current = d; })

    svg_inner = svg.append('g')
                   .attr('transform', translate(
                        (width / 2) + offset_info_x, (radius + margin) + offset_inner_y))

    svg_inner.append("text")
             .style('text-anchor', 'middle')
             .style('alignment-baseline', 'middle')
             .style('text-rendering', 'optimizeLegibility')
             .attr('y', inner_title_y)
             .text(metadata.inner_title)
             .attr('class', 'inner_title')

    var inner = svg_inner.append("text")
                         .style('text-anchor', 'middle')
                         .style('alignment-baseline', 'middle')
                         .style('text-rendering', 'optimizeLegibility')
                         .attr('class', 'inner_title')

    inner.append("tspan")
         .attr('class', 'inner_value')
         .text(metadata.inner_value)

    inner.append("tspan")
         .attr('class', 'inner_units')
         .attr('dx', 3)
         .text(metadata.inner_units)

    svg_main.append("text")
            .attr('y', radius + legend_title_height + margin)
            .attr('height', legend_title_height)
            .attr('class', 'legend_title')
            .style('text-anchor', 'middle')
            .text(metadata.title)

    legend = svg_main.selectAll('.legend')
                     .data(function() { return colors_main.domain() })
                     .enter()
                     .append('g')
                     .attr('class', 'legend')
                     .attr('transform', function(d, i) {
                         return translate(
                            - width / 2,
                            radius + legend_title_height + legend_title_top_margin + legend_title_bottom_margin + i * legend_row_height
                         )})

    legend.append('rect')
              .attr('width', legend_rect_size)
              .attr('height', legend_rect_size)
              .attr('y', - legend_rect_size)
              .attr('x', legend_rect_left_margin)
              .attr('rx', 5)
              .attr('ry', 5)
              .style('fill', colors_main)
              .style('stroke', colors_main)

    legend.append('text')
          .attr('x', legend_rect_size + legend_rect_right_margin + legend_rect_left_margin)
          .text(function(d) { return d; })

    legend.append('text')
          .attr('x', (width - 2 * margin - 10))
          .attr('class', 'legend_desc')
          .style('text-anchor', 'end')
          .style('fill', '#989898')
          .text(function(d) {
                    for (var i = 0; i < values_main.length; i++)
                        if (values_main[i].label == d)
                            return values_main[i].desc }
                )

    donuts.push({'pie': pie_main, 'path': path_main, 'legend': legend})
    return donuts
}

function translate(x, y) {
    return 'translate(' + x + ',' + y + ')'
}

function arc_tween(a) {
    var i = d3.interpolate(this._current, a)
    this._current = i(0)
    var arc = this._arc
    return function(t) {
        return arc(i(t));
    }
}
